#ifndef _SHADE_H
#define _SHADE_H
#include "matrix.h"

/**** Defines *********************/

#define MAXLINE 80

#define ASCII 0
#define BINARY 1

#define FALSE 0
#define TRUE 1


/**** Structures and types ********
 **** stolen and modified from poly.h *******/
typedef char bool;
typedef int file_t;

typedef struct {		/* A POLYGON VERTEX */
    double sx, sy, sz, sw;	/* screen space position (sometimes homo.) */
    double x, y, z;		/* world space position */
    vector normal;
    bool shaded;
    double r, g, b;		/* (red,green,blue) color */
} Poly_vert;

typedef struct{
  int v[3];          /* vertices */
  unsigned long mask;     /* interpolation mask for vertex elems */
  int n;                    /* number of sides */
  vector normal;          /* normal (used for back face culling) */
  int degenerate;         /* 2 or more of vertices are the same */
}Triangle;

typedef struct {		/* WINDOW: A DISCRETE 2-D RECTANGLE */
    int x0, y0;			/* xmin and ymin */
    int x1, y1;			/* xmax and ymax (inclusive) */
} Window;

typedef struct{     /* For Z-buffer hidden surface determination */  
  double y;
}Y_struct;

struct tree_struct {
  struct tree_struct *left, *right;
  Window* win;
  Y_struct** buffer;
  int xsize, ysize;
};

typedef struct tree_struct tree;

/****** Function Prototypes *******/
#ifdef ANSI
void Read_binary_polygon_data(char *DataFileName,
			      int *Num_Vertices, double ***Vertices,
			      int *Num_Polygons, int ***Polygons);
void Read_ascii_polygon_data(char *DataFileName,
			     int *Num_Vertices, double ***Vertices,
			     int *Num_Polygons, int ***Polygons);
void Read_binary_color(char *DataFileName,
		       int *Num_Vertices, double ***Vertices);
void Read_ascii_color(char *DataFileName,
		      int *Num_Vertices, double ***Vertices);

extern void myerror(int errorno);
pixelproc(int x, int z, Poly_vert* pnt);
#else
void Read_binary_polygon_data();
void Read_ascii_polygon_data();
void Read_binary_color();
void Read_ascii_color();
extern void myerror();
extern pixelproc();
extern bool notInShadow();
extern void allocateBuffer();
extern void addToTree();
extern void ScanTree();
tree* mkNewTree();

#endif /* ANSI */

void poly_scan();
#endif














